@@ -11,17 +11,22 @@ RSpec.describe "bundle install with a lockfile present" do
     install_gemfile(gf)
   end
 
+  it "touches the lockfile on install even when nothing has changed" do
+    subject
+    expect { bundle :install }.to change { bundled_app_lock.mtime }
+  end
+
   context "gemfile evaluation" do
     let(:gf) { super() + "\n\n File.open('evals', 'a') {|f| f << %(1\n) } unless ENV['BUNDLER_SPEC_NO_APPEND']" }
 
     context "with plugins disabled" do
       before do
-        bundle! "config set plugins false"
+        bundle "config set plugins false"
         subject
       end
 
       it "does not evaluate the gemfile twice" do
-        bundle! :install
+        bundle :install
 
         with_env_vars("BUNDLER_SPEC_NO_APPEND" => "1") { expect(the_bundle).to include_gem "rack 1.0.0" }
 
@@ -31,10 +36,10 @@ RSpec.describe "bundle install with a lockfile present" do
       end
 
       context "when the gem is not installed" do
-        before { FileUtils.rm_rf ".bundle" }
+        before { FileUtils.rm_rf bundled_app(".bundle") }
 
         it "does not evaluate the gemfile twice" do
-          bundle! :install
+          bundle :install
 
           with_env_vars("BUNDLER_SPEC_NO_APPEND" => "1") { expect(the_bundle).to include_gem "rack 1.0.0" }
 
