@@ -1,22 +1,24 @@
 # frozen_string_literal: true
-require 'rubygems/command_manager'
+
+require "rubygems/command_manager"
 
 ##
 # This is an example of exactly what NOT to do.
 #
 # DO NOT include code like this in your rubygems_plugin.rb
 
-class Gem::Commands::InterruptCommand < Gem::Command
+module Gem::Commands
+  remove_const(:InterruptCommand) if defined?(InterruptCommand)
+end
 
+class Gem::Commands::InterruptCommand < Gem::Command
   def initialize
-    super('interrupt', 'Raises an Interrupt Exception', {})
+    super("interrupt", "Raises an Interrupt Exception", {})
   end
 
   def execute
     raise Interrupt, "Interrupt exception"
   end
-
 end
 
 Gem::CommandManager.instance.register_command :interrupt
-
